package dev.fyloz.trial.colorrecipesexplorer.web.controller;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.JSONResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Company;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.services.PasswordService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.INDEX;
import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.PASSWORD_VALIDATION;

@Controller
public class IndexController {

    private RecipeService recipeService;

    @Autowired
    public IndexController(RecipeService recipeService) {
        this.recipeService = recipeService;
    }

    /**
     * Affiche la page d'acceuil.
     *
     * @return La page à afficher.
     */
    @GetMapping({INDEX, "/"})
    public ModelAndView getPage() {
        return new ModelResponseBuilder(INDEX)
                .addResponseData(ResponseDataType.RECIPE_MAP, recipeService.getRecipesByCompany())
                .build();
    }

    //    @GetMapping(value = SEARCH, produces = MediaType.APPLICATION_JSON_VALUE)
//    @ResponseBody
    public Map<String, Object> searchWord(@RequestParam String searchString) {
        Map<Company, List<Recipe>> searchResult = recipeService.getRecipesForSearchString(searchString);
        Map<Integer, List<Integer>> outputResult = new HashMap<>();

        for (Company c : searchResult.keySet()) {
            outputResult.put(c.getCompanyID(), searchResult.get(c).stream().map(Recipe::getRecipeID).collect(Collectors.toList()));
        }

        return new JSONResponseBuilder()
                .addAttribute("result", outputResult)
                .build();
    }

    /**
     * Valide un mot de passe reçu dans une requête HTTP POST, dans le champ "password".
     *
     * @param data Corps de la requête HTTP
     * @return Si le mot de passe est valide.
     */
    @PostMapping(value = PASSWORD_VALIDATION, consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public boolean validatePassword(@RequestBody Map<String, Object> data) {
        return PasswordService.isValid((String) data.get("password"));
    }
}
