package dev.fyloz.trial.colorrecipesexplorer.web.controller;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Mix;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MixService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MixTypeService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import dev.fyloz.trial.colorrecipesexplorer.core.utils.FileUtils;
import org.commonmark.node.Node;
import org.commonmark.parser.Parser;
import org.commonmark.renderer.html.HtmlRenderer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ResourceLoader;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import java.util.Collection;
import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.StringBank.MATERIALS;
import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.*;

@Controller
public class OthersController {

    private RecipeService recipeService;
    private MaterialService materialService;
    private MixTypeService mixTypeService;
    private MixService mixService;
    private ResourceLoader resourceLoader;

    @Autowired
    public OthersController(RecipeService recipeService, MaterialService materialService, MixTypeService mixTypeService, MixService mixService, ResourceLoader resourceLoader) {
        this.recipeService = recipeService;
        this.materialService = materialService;
        this.mixTypeService = mixTypeService;
        this.mixService = mixService;
        this.resourceLoader = resourceLoader;
    }

    @GetMapping(CLOSE_TAB)
    public ModelAndView closeTab() {
        return new ModelResponseBuilder(CLOSE_TAB).build();
    }

    @GetMapping(MATERIAL_SELECTOR_MIX)
    public ModelAndView getMaterialSelectorFragment(@PathVariable int recipeID, @PathVariable int mixID) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder(MATERIAL_SELECTOR_FRAGMENT);

        Optional<Recipe> optionalRecipe = recipeService.getByID(recipeID);
        if (optionalRecipe.isEmpty()) {
            return modelResponseBuilder
                    .withView("")
                    .build();
        }

        Optional<Mix> optionalMix = mixService.getByID(mixID);
        boolean mixExist = optionalMix.isPresent();

        Collection<Material> materials = mixExist ? mixService.getAvailableMaterialsForMix(optionalRecipe.get(), optionalMix.get()) : mixService.getAvailableMaterialsForNewMix(optionalRecipe.get());

        return modelResponseBuilder
                .addAttribute(MATERIALS, materials)
                .build();
    }

    @GetMapping(value = UPDATES)
    public ModelAndView showUpdates() {
        return new ModelResponseBuilder(UPDATES).build();
    }

    @GetMapping(value = UPDATES_GET, produces = MediaType.TEXT_HTML_VALUE)
    @ResponseBody
    public ResponseEntity<String> getUpdates() {
        String fileContent = FileUtils.readClasspathFile(resourceLoader, "classpath:updates.md");
        if (fileContent == null) return ResponseEntity.status(HttpStatus.NOT_FOUND).build();

        Parser parser = Parser.builder().build();
        Node document = parser.parse(fileContent);
        HtmlRenderer renderer = HtmlRenderer.builder().build();

        return new ResponseEntity<>(renderer.render(document), HttpStatus.OK);
    }
}
