package dev.fyloz.trial.colorrecipesexplorer.web.controller.creators;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialTypeService;
import dev.fyloz.trial.colorrecipesexplorer.core.utils.ControllerUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;
import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.CREATOR_MATERIAL_TYPE;
import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.INDEX;

@Controller
public class MaterialTypeCreatorController {

    private MaterialTypeService materialTypeService;

    @Autowired
    public MaterialTypeCreatorController(MaterialTypeService materialTypeService) {
        this.materialTypeService = materialTypeService;
    }

    @GetMapping(CREATOR_MATERIAL_TYPE)
    public ModelAndView getPage(ModelAndView model, MaterialType materialType) {
        return new ModelResponseBuilder(model)
                .withView(CREATOR_MATERIAL_TYPE)
                .addResponseData(ResponseDataType.MATERIAL_TYPE, materialType == null ? new MaterialType() : materialType)
                .build();
    }

    @PostMapping(CREATOR_MATERIAL_TYPE)
    public ModelAndView create(@Valid MaterialType materialType) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder(ControllerUtils.redirect(INDEX));

        if (materialTypeService.isValidForCreation(materialType)) {
            Optional<MaterialType> optionalMaterialType = materialTypeService.save(materialType);
            if (optionalMaterialType.isPresent()) {
                return getPage(modelResponseBuilder
                        .addResponseCode(ResponseCode.SUCCESS_SAVING_MATERIAL_TYPE, optionalMaterialType.get().getMaterialTypeName())
                        .build(), null);
            } else {
                modelResponseBuilder.addResponseCode(ResponseCode.ERROR_SAVING);
            }
        } else {
            modelResponseBuilder.addResponseCode(ResponseCode.MATERIAL_TYPE_ALREADY_EXIST, materialType.getMaterialTypeName());
        }

        return getPage(modelResponseBuilder.build(), materialType);
    }
}
