package dev.fyloz.trial.colorrecipesexplorer.web.controller.files;

import dev.fyloz.trial.colorrecipesexplorer.core.io.file.FileHandler;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;

import javax.servlet.http.HttpServletRequest;
import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.CLOSE_TAB;
import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.SIMDUT_FILES;

@Controller
public class SIMDUTFilesController {

    private MaterialService materialService;

    @Autowired
    public SIMDUTFilesController(MaterialService materialService) {
        this.materialService = materialService;
    }

    @GetMapping(SIMDUT_FILES)
    public ResponseEntity<byte[]> getFile(HttpServletRequest request, @PathVariable int materialID) {
        Optional<Material> optionalMaterial = materialService.getByID(materialID);
        HttpHeaders headers = new HttpHeaders();

        if (optionalMaterial.isEmpty()) {
            headers.add("Location", request.getHeader("referer"));
            return new ResponseEntity<>(headers, HttpStatus.FOUND);
        }

        FileHandler fileHandler = new FileHandler(String.format("%s_%s", materialID, optionalMaterial.get().getMaterialCode()), FileHandler.FileContext.SIMDUT, FileHandler.FileExtension.PDF);
        if (!fileHandler.isValid()) {
            headers.add("Location", "/" + CLOSE_TAB);
            return new ResponseEntity<>(headers, HttpStatus.FOUND);
        }

        byte[] fileContent = fileHandler.readFile();
        headers.setContentType(MediaType.APPLICATION_PDF);
        return new ResponseEntity<>(fileContent, headers, HttpStatus.OK);
    }

    @PostMapping(SIMDUT_FILES)
    public ResponseEntity<Void> getFile(@PathVariable int materialID) {
        Optional<Material> optionalMaterial = materialService.getByID(materialID);
        if (optionalMaterial.isEmpty()) return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();

        FileHandler fileHandler = new FileHandler(String.format("%s_%s", materialID, optionalMaterial.get().getMaterialCode()), FileHandler.FileContext.SIMDUT, FileHandler.FileExtension.PDF);
        if (!fileHandler.isValid()) return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        return ResponseEntity.status(HttpStatus.FOUND).build();
    }
}
